/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef _ds485_h
#define _ds485_h

/**
 * Status des dS485 Bus
 */
typedef enum {
    DS485S_STATE_ISOLATED = 1,
    DS485S_STATE_JOIN,
    DS485S_STATE_CONNECTED,
    DS485S_STATE_ACTIVE,
} ds485s_bus_state_t;

/**
 * Internal errors
 */
enum {
    DS485_INTERNAL_ERROR_OK = 0,
    DS485_INTERNAL_ERROR_GENERAL = -1,
    DS485_INTERNAL_ERROR_EMPTY_TX_QUEUE = -2,
    DS485_INTERNAL_ERROR_INVALID_BUS_ADDR = -3,
    DS485_INTERNAL_ERROR_ERANGE = -4,
};

/**
 * dS485 Protocol Statistic
 */
typedef struct {
    /* TX */
    uint32_t txToken;
    uint16_t txStartupRequest;
    uint16_t txStartupResponse;
    uint16_t txJoinRequest;
    uint16_t txJoinResponse;
    uint16_t txGetAddressRequest;
    uint16_t txGetAddressResponse;
    uint16_t txSetAddressRequest;
    uint16_t txSetAddressResponse;
    uint16_t txSetSuccessorAddressRequest;
    uint16_t txSetSuccessorAddressResponse;
    uint16_t txResponse;
    uint16_t txDataRequest;
    uint16_t txDataResponse;
    uint16_t txDataEvent;
    /* RX */
    uint16_t rxResponse;
    uint16_t rxDataRequest;
    uint16_t rxDataResponse;
    uint16_t rxDataEvent;
    /* Actions */
    uint16_t startRegisterStation;
    uint16_t doneRegisterStation;
    uint16_t enterOperational;
    uint16_t successorLost;
} DS485Statistics;

/***
 * External references - have to be supplied by platform module
 */

/**
 * Receive frame error bits
 */
#define RX_FLAG_CRC_ERROR            0x01
#define RX_FLAG_FRAME_ERROR            0x02
#define RX_FLAG_PROTOCOL_ERROR        0x04
#define RX_FLAG_NOTFORME            0x10
#define RX_FLAG_INVALID_ADDRESS        0x80

#define RX_FLAG_ERRORS (RX_FLAG_CRC_ERROR | RX_FLAG_FRAME_ERROR | RX_FLAG_PROTOCOL_ERROR)

#endif
